@@ -1,5 +1,5 @@
 //#include <stdlib.h>
-#include <float.h>
+#include <cfloat>
 
 #include "general.h"
 #include "error.h"
@@ -7,6 +7,8 @@
 #include "contain.h"
 #include "options.h"
 
+using namespace std ;
+
 int NoEstimators = 37;
 estDsc estName[38]={
     {"", ""} ,
